@@ -216,10 +216,10 @@ def crop_bounds(images, apply=True, crop_global=True, background=0):
     if not isinstance(images, list):
         images = [images]
     nzs = [np.where(np.sum(np.atleast_3d(im) != background, axis=2) > 0) for im in images]
-    xmins = [np.min(nz[1]) if len(nz[1]) else 0 for nz in nzs]
-    xmaxs = [np.max(nz[1]) + 1 if len(nz[1]) else im.shape[1] for nz, im in zip(nzs, images)]  # +1 to allow easier indexing
-    ymins = [np.min(nz[0]) if len(nz[0]) else 0 for nz in nzs]
-    ymaxs = [np.max(nz[0]) + 1 if len(nz[0]) else im.shape[0] for nz, im in zip(nzs, images)]  # +1 to allow easier indexing
+    xmins = [np.min(nz[1]) if len(nz[1]) else im.shape[1] // 2 for nz, im in zip(nzs, images)]
+    xmaxs = [np.max(nz[1]) + 1 if len(nz[1]) else im.shape[1] // 2 for nz, im in zip(nzs, images)]  # +1 to allow easier indexing
+    ymins = [np.min(nz[0]) if len(nz[0]) else im.shape[0] // 2 for nz, im in zip(nzs, images)]
+    ymaxs = [np.max(nz[0]) + 1 if len(nz[0]) else im.shape[0] // 2 for nz, im in zip(nzs, images)]  # +1 to allow easier indexing
     if crop_global:
         # fix cropping boundaries for all images
         xmins = [np.min(xmins) for _ in xmins]
@@ -388,11 +388,11 @@ def write_openexr(filename: str, image: np.ndarray, channels: list = None, pixel
             channels = ['L']
         elif num_channels == 3:
             channels = ['R', 'G', 'B']
-        elif num_channels == 3:
+        elif num_channels == 4:
             channels = ['R', 'G', 'B', 'A']
         else:
             num_digits = max(1, int(np.ceil(np.log10(num_channels + 1))))
-            channels = [('ch%0' + str(num_digits) + 'd') % ci for ci in nrange(num_channels)]
+            channels = [('ch%0' + str(num_digits) + 'd') % ci for ci in range(num_channels)]
     assert len(channels) == num_channels, 'number of channels (%d) must match the image ' \
                                           'dimensions (%d)' % (len(channels), num_channels)
     assert len(pixel_types) == num_channels, 'pixel type was speficied %d times, should match the image ' \
